/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.

 Christian Herdtweck, Intra2net AG 2015
 Based on an example in Boost Documentation (by Christopher M. Kohlhoff)
 and adaptation by Guilherme M. Ferreira
 */

#include "icmp/icmpechodata.h"

#include "boost/date_time/posix_time/posix_time_types.hpp"
#include "boost/date_time/time_resolution_traits.hpp"
#include <logfunc.hpp>

using boost::date_time::time_resolution_traits_adapted64_impl;
using I2n::Logger::GlobalLogger;


IcmpEchoData::IcmpEchoData(const uint16_t identifier,
                           const uint16_t sequence_number,
                           const std::string &optional_data)
    : IcmpData( 4 + optional_data.length() )
{
    IcmpData::raw_data.encode16(0, 1, identifier);
    IcmpData::raw_data.encode16(2, 3, sequence_number);
    IcmpData::raw_data.encode_string(4, optional_data);
    GlobalLogger.debug() << "Done creating echo request" << std::endl;
}

IcmpEchoData::IcmpEchoData(const std::size_t size_arg)
    : IcmpData( size_arg )
{}


bool IcmpEchoData::match_destination_unreachable(
                                    const uint16_t identifier,
                                    const uint16_t sequence_number,
                                    const address &destination_address) const
{ return false; }


bool IcmpEchoData::match_time_exceeded(
                                    const uint16_t identifier,
                                    const uint16_t sequence_number,
                                    const address &destination_address) const
{ return false; }


bool IcmpEchoData::match_echo_reply(const uint16_t identifier,
                                    const uint16_t sequence_number) const
{
    return get_identifier() == identifier
        && get_sequence_number() == sequence_number;
}

uint16_t IcmpEchoData::get_identifier() const
{ return IcmpData::raw_data.decode16(0, 1);  }

uint16_t IcmpEchoData::get_sequence_number() const
{ return IcmpData::raw_data.decode16(2, 3);  }

/**
 * @brief Prints the ICMP echo reply messages.
 *
 * @return void
 */
void IcmpEchoData::print( const size_t &bytes_received,
            const boost::posix_time::ptime &time_packet_sent,
            const std::string &remote_address,
            const uint32_t ttl) const
{
    boost::posix_time::ptime now
                      = boost::posix_time::microsec_clock::universal_time();
    time_resolution_traits_adapted64_impl::int_type elapsed_time =
            (now - time_packet_sent).total_milliseconds();

    GlobalLogger.info() << bytes_received << " bytes "
         << "from " << remote_address
         << ": icmp_seq=" << get_sequence_number()
         << " ttl=" << ttl
         << " time=" << elapsed_time << " ms" << std::endl;
}

std::string IcmpEchoData::to_string() const
{
    std::stringstream buf;
    buf << "[EchoData ID=" << std::showbase << std::hex << get_identifier()
        << ",seq.nr=" << std::noshowbase << std::dec<< get_sequence_number()
        << ",data of size " << IcmpData::size-4;
    //for (int idx=4; idx < IcmpData::size; ++idx)
    //    buf << IcmpData::raw_data[idx];
    buf << "]";
    return buf.str();
}

